import { isNil, splitEvery } from 'ramda';
import { FC, useMemo } from 'react';
import tw, { css, styled } from 'twin.macro';
import { borders } from '../../constants/border';
import { DisplayConfig } from '../models/display-config';
import { LedStripConfig } from '../models/led-strip-config';
import { ScreenshotDto } from '../models/screenshot.dto';
import { LedStrip } from './led-strip';
import { StyledPixel } from './styled-pixel';

interface CompletedLedStripProps {
  screenshots: ScreenshotDto[];
  onDisplayConfigChange?: (value: DisplayConfig) => void;
}

type BorderLedStrip = {
  pixels: [number, number, number][];
  config: LedStripConfig | null;
};

const StyledContainer = styled.section(
  ({ rows, columns }: { rows: number; columns: number }) => [
    tw`grid m-4 pb-2`,
    css`
      grid-template-columns: repeat(${columns}, 1fr);
      grid-template-rows: auto repeat(${rows}, 1fr);
    `,
  ],
);
const StyledCompletedContainer = styled.section(
  tw`dark:bg-transparent shadow-xl border-gray-500 border rounded-full flex flex-wrap justify-around items-center mb-2`,
  css`
    grid-column: 1 / -1;
  `,
);

export const CompletedLedStrip: FC<CompletedLedStripProps> = ({
  screenshots,
  onDisplayConfigChange,
}) => {
  const borderLedStrips: BorderLedStrip[] = useMemo(() => {
    return screenshots.flatMap((ss) =>
      borders.map((b) => ({
        pixels: splitEvery(3, Array.from(ss.colors[b])) as [number, number, number][],
        config: ss.config.led_strip_of_borders[b],
      })),
    );
  }, [screenshots]);
  const ledCount = useMemo(
    () => borderLedStrips.reduce((prev, curr) => prev + curr.pixels.length, 0),
    [borderLedStrips],
  );

  const completedPixels = useMemo(() => {
    const completed: [number, number, number][] = new Array(ledCount).fill([0, 0, 0]);
    borderLedStrips.forEach(({ pixels, config }) => {
      if (isNil(config)) {
        return;
      }
      if (config.global_start_position <= config.global_end_position) {
        pixels.forEach((color, i) => {
          completed[config.global_start_position + i] = color;
        });
      } else {
        pixels.forEach((color, i) => {
          completed[config.global_start_position - i] = color;
        });
      }
    });

    return completed.map((color) => <StyledPixel rgb={color} />);
  }, [ledCount, borderLedStrips]);

  const strips = useMemo(() => {
    return borderLedStrips.map(({ config, pixels }, index) => (
      <LedStrip
        key={index}
        colors={Uint8Array.from(pixels.flat())}
        config={config}
        css={css`
          grid-column-start: ${(config?.global_start_position ?? 0) + 1};
          grid-column-end: ${(config?.global_end_position ?? 0) + 1};
        `}
      />
    ));
  }, [borderLedStrips]);
  return (
    <StyledContainer rows={screenshots.length * borders.length} columns={ledCount}>
      <StyledCompletedContainer>{completedPixels}</StyledCompletedContainer>
      {strips}
    </StyledContainer>
  );
};
